/* Copyright (c) 2009 Matthias Kaeppler
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package oauth.signpost.exception;

@SuppressWarnings("serial")
public class OAuthCommunicationException extends OAuthException {

    private String responseBody;
    
    public OAuthCommunicationException(Exception cause) {
        super("Communication with the service provider failed: "
                + cause.getLocalizedMessage(), cause);
    }
    
    public OAuthCommunicationException(String message, String responseBody) {
        super(message);
        this.responseBody = responseBody;
    }
    
    public String getResponseBody() {
        return responseBody;
    }
}
